//
//  oh_hw1_delayViewController.m
//  oh_hw1_delay
//
//  Created by Jieun Oh on 1/9/10.
//  Copyright Stanford University 2010. All rights reserved.
//

#import "oh_hw1_delayViewController.h"
#import "mo_audio.h"
#import "mo_touch.h"
#import "mo_def.h"
#import "Delay.h"
#import <cstdlib>

#import <math.h>


#define SRATE 44100
#define FRAMESIZE 128
#define NUMCHANNELS 2

Delay g_d(1000);


void delayCallback (Float32 * buffer, UInt32 framesize, void* userData)
{
	
	for (int i=0; i<framesize; i++)
	{
		float popped = g_d.getNextSample();
		g_d.writeToDelay ( buffer[2*i] + g_d.getGain()*popped ); 
		buffer[2*i] = buffer[2*i+1] = popped;
	}
}


// Implement audio callback here
void audioCallback( Float32 * buffer, UInt32 framesize, void* userData)
{
	delayCallback( buffer, framesize, userData);
}

// Implement accel callback here
void touchCallback ( NSSet * touches, UIView * view, const std::map<int, MoTouchTrack> & touchPts, void * data)
{
	
	UILabel* delayTime = (UILabel*) data;
	
	long count = [touches count];
	
	if (count==2) // two fingers
	{
		CGPoint pt1, pt2;
		bool second = false;
		
		for (UITouch * touch in touches)
		{
			if (!second) {pt1 = [touch locationInView:view]; second = true;}
			else {pt2 = [touch locationInView:view];}
		}
		
		float sqdistance = pow(pt1.x - pt2.x, 2) + pow(pt1.y - pt2.y, 2) ;
		// NSLog(@"sqdistance: %f", sqdistance);
		
		float samplesDelayed = pow((sqdistance/3000.0),2)*10;  
		g_d.setDelayLen( (int)samplesDelayed ); 
		
		float secDelayed = samplesDelayed / SRATE;
		[delayTime setText:[NSString stringWithFormat:@"%f", secDelayed]];
		
	}
	
}




@implementation oh_hw1_delayViewController

@synthesize myG;
@synthesize mySlider;
@synthesize myDelayTime;

-(IBAction)moveSlider:(UISlider *)sender
{
	g_d.setGain((float) sender.value);
	[myG setText:[NSString stringWithFormat:@"%f", sender.value]];
}


/*
// The designated initializer. Override to perform setup that is required before the view is loaded.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
    }
    return self;
}
*/

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/



// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
	
	
	NSLog(@"beginning of viewDidLoad");
	
	// init audio
	NSLog(@"Initializing Audio");
	
	// init the audio layer
	bool result = MoAudio::init(SRATE, FRAMESIZE, NUMCHANNELS);
	
	if (!result)
	{
		NSLog(@"cannot initialize real-time audio!");
		return;
	}
	
	// start the audio layer, registering a callback method
	result = MoAudio::start( audioCallback, NULL);
	if (!result)
	{
		NSLog(@"cannot start real-time audio!");
		return;
	}
	
	
	// init touch
	MoTouch::instance()->addCallback( touchCallback, myDelayTime);
	 
}



/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}

@end
