/*
 *  delay.cpp
 *  oh_hw1_delay
 *
 *  Created by Jieun Oh on 1/9/10.
 *  Copyright 2010 Stanford University. All rights reserved.
 *
 */

#import "Delay.h"
#import <cstdlib>
#import <memory.h>
#import <assert.h>
#import <math.h>


// ctor
Delay::Delay()
{
	init( 1000);
}

Delay::Delay( int samples)
{
	init (samples);
}


void Delay::init( int samples)
{
	// read and write indices
    myReader = 0;
    myWriter = 0;
	myBufferLen = 100000; 
	myGain = 0.5; 
	
    setDelayLen( samples); 
	
    // (dynamically) allocate the delay buffer
    myDelay = new SAMPLE[myBufferLen];
	cerr << "myDelay address" << myDelay << endl;	
	
    // zero out
    memset( myDelay, 0, myBufferLen*sizeof(SAMPLE) );

    assert( myDelayLen <= myBufferLen );
		
    // set indices
    myReader = myWriter - myDelayLen;
	
    // wrap it properly
    if( myReader < 0 ) myReader += myBufferLen;
}

// dtor
Delay::~Delay()
{
	delete [] myDelay;
}



// reads the next sample from myDelay
SAMPLE Delay::getNextSample()
{
	SAMPLE value = myDelay[myReader];
	myReader++;
	if (myReader == myBufferLen ) {
		myReader=0; 
	}
	return value; 
}


// sets how many samples are delayed 
void Delay::setDelayLen(int samples) 
{
	myDelayLen = samples;
	
	myReader = myWriter - myDelayLen;
    if( myReader < 0 ) myReader += myBufferLen;
}


// populates myDelay vector

void Delay::writeToDelay( SAMPLE s)
{	
	// print(); --> crashes cause can't access myDelay[i]
	myDelay[myWriter] = s;
	
	// increment
	myWriter++;
		
	// wrap indices
	myWriter %= myBufferLen;
}
 


// for debugging
void Delay::print()
{
	for (int i=0; i<10; i++) {
		cerr << "address of myDelay[" << i << "] is: " << myDelay+i << endl; 
		cerr << "myDelay[" << i << "] = " << myDelay[i] << endl;
	}
}


void Delay::setGain( float g)
{
	myGain = g;
}

float Delay::getGain()
{
	return myGain;
}



