//-----------------------------------------------------------------------------
// name: mo_touch.mm
// desc: MoPhO API for multi-touch
//
// authors: Jieun Oh
//          Nick Bryan
//          Jorge Herrera
//          Ge Wang
//
//    date: Fall 2009
//    version: 0.1
//
// Stanford Mobile Phone Orchestra
//     http://mopho.stanford.edu/
//-----------------------------------------------------------------------------
#include "mo_touch.h"
#import <objc/objc.h>
#import <objc/runtime.h>
#import <objc/message.h>


//-----------------------------------------------------------------------------
// name: touchesBegan()
// desc: handles the start of a touch
//-----------------------------------------------------------------------------
void myTouchesBegan(id self, SEL _cmd, NSSet * touches, UIEvent * event)
{
    NSSet * allTouches = [event allTouches];
    MoTouch::instance()->update( allTouches, self );    
}


//-----------------------------------------------------------------------------
// name: touchesMoved()
// desc: handles the continuation of a touch
//-----------------------------------------------------------------------------
void myTouchesMoved(id self, SEL _cmd, NSSet * touches, UIEvent *event)
{   
    NSSet * allTouches = [event allTouches];
    MoTouch::instance()->update( allTouches, self );
}


//-----------------------------------------------------------------------------
// name: touchesEnded()
// desc: handles the end of a touch event when the touch is a tap
//-----------------------------------------------------------------------------
void myTouchesEnded(id self, SEL _cmd, NSSet *touches, UIEvent * event)
{
    NSSet * allTouches = [event allTouches];
    MoTouch::instance()->update( allTouches, self );
}


//-----------------------------------------------------------------------------
// name: touchesCancelled()
// desc: handles the end of a touch event
//-----------------------------------------------------------------------------
void myTouchesCancelled(id self, SEL _cmd, NSSet *touches, UIEvent * event)
{
    NSSet * allTouches = [event allTouches];
    MoTouch::instance()->update( allTouches, self );
}


//-----------------------------------------------------------------------------
// name: instance()
// desc: get instance
//-----------------------------------------------------------------------------
MoTouch * MoTouch::ourInstance = NULL;
MoTouch * MoTouch::instance()
{
    if( ourInstance == NULL )
    {
        ourInstance = new MoTouch();
        // manually override the touch methods
        ourInstance->setup();
    }
    
    return ourInstance;
}


MoTouch::MoTouch() 
{ }


MoTouch::~MoTouch()
{ }


void MoTouch::setup()
{	
    //Override the touches methods, so the user doesn't have to do it
    Method method = class_getInstanceMethod([UIView class], @selector(touchesBegan:withEvent:));
    method_setImplementation(method, (IMP)myTouchesBegan);
    
    method = class_getInstanceMethod([UIView class], @selector(touchesMoved:withEvent:));
    method_setImplementation(method, (IMP)myTouchesMoved);
    
    method = class_getInstanceMethod([UIView class], @selector(touchesEnded:withEvent:));
    method_setImplementation(method, (IMP)myTouchesEnded);
    
    method = class_getInstanceMethod([UIView class], @selector(touchesCancelled:withEvent:));
    method_setImplementation(method, (IMP)myTouchesCancelled);
}


void MoTouch::addCallback( const MoTouchCallback & callback, void * data )
{
    m_clients.push_back( callback );
    m_clientData.push_back( data );
}


void MoTouch::update( NSSet * touches, UIView * view )
{
    CGRect bounds = [view bounds];
    for( UITouch * touch in touches )
    {
        CGPoint location;
        
        // convert touch point from UIView referential to OpenGL one (upside-down flip)
        location = [touch locationInView:view];
        location.y = bounds.size.height - location.y;
        
        // cast touch pointer to int
        long key = (long) touch;
        
        // began
        if( touch.phase == UITouchPhaseBegan )
        {
            // look up key
            if( m_touchPts.find( key ) == m_touchPts.end() )
            {
                m_touchPts[key].touch = touch; 
            }
        }
        // stationary
        else if( touch.phase == UITouchPhaseStationary )
        {
            m_touchPts[key].touch = touch; 
        }
        // moved
        else if( touch.phase == UITouchPhaseMoved )
        {
            // update location
            m_touchPts[key].touch = touch; 
        }
        // ended or cancelled
        else if( touch.phase == UITouchPhaseEnded || touch.phase == UITouchPhaseCancelled )
        {
            // look up key
            if( m_touchPts.find( key ) != m_touchPts.end() )
            {
                m_touchPts.erase( key );
            }
        }
    }
    
    for (int i=0; i<m_clients.size(); i++) 
    {
        m_clients[i]( touches, view, m_touchPts, m_clientData[i] );
    }	
}
